@@ -36,6 +36,7 @@ UNMutableNotificationContent = ObjCClass("UNMutableNotificationContent")
 UNNotificationRequest = ObjCClass("UNNotificationRequest")
 UNNotificationAction = ObjCClass("UNNotificationAction")
 UNNotificationCategory = ObjCClass("UNNotificationCategory")
+UNNotificationSound = ObjCClass("UNNotificationSound")
 
 NSSet = ObjCClass("NSSet")
 
@@ -68,15 +69,13 @@ class NotificationCenterDelegate(NSObject):  # type: ignore
     ) -> None:
 
         # Get the notification which was clicked from the platform ID.
-        internal_nid = py_from_ns(
-            response.notification.request.content.userInfo["internal_nid"]
-        )
-        notification = self.interface.current_notifications[internal_nid]
+        platform_nid = py_from_ns(response.notification.request.identifier)
+        py_notification = self.interface._notification_for_nid[platform_nid]
 
         # Get and call the callback which corresponds to the user interaction.
         if response.actionIdentifier == UNNotificationDefaultActionIdentifier:
 
-            callback = notification.action
+            callback = py_notification.action
 
             if callback:
                 callback()
@@ -85,7 +84,7 @@ class NotificationCenterDelegate(NSObject):  # type: ignore
 
             action_id_str = py_from_ns(response.actionIdentifier)
 
-            callback = notification.buttons.get(action_id_str)
+            callback = py_notification.buttons.get(action_id_str)
 
             if callback:
                 callback()
@@ -102,25 +101,34 @@ class CocoaNotificationCenter(DesktopNotifierBase):
 
     :param app_name: The name of the app. Does not have any effect because the app
         name is automatically determined from the bundle or framework.
+    :param app_icon: The icon of the app. Does not have any effect because the app
+        icon is automatically determined from the bundle or framework.
+    :param notification_limit: Maximum number of notifications to keep in the system's
+        notification center.
     """
 
-    _notification_categories: Dict[Tuple[str, ...], str]
-
-    def __init__(self, app_name: str) -> None:
-        super().__init__(app_name)
+    def __init__(
+        self,
+        app_name: str = "Python",
+        app_icon: Optional[str] = None,
+        notification_limit: Optional[int] = None,
+    ) -> None:
+        super().__init__(app_name, app_icon, notification_limit)
         self.nc = UNUserNotificationCenter.currentNotificationCenter()
         self.nc_delegate = NotificationCenterDelegate.alloc().init()
         self.nc_delegate.interface = self
-
-        if self.nc:
-            self.nc.delegate = self.nc_delegate
-        else:
-            logger.warning("No notification center available")
+        self.nc.delegate = self.nc_delegate
 
         self._did_request_authorisation = False
-        self._notification_categories = {}
+        self._notification_categories: Dict[Tuple[str, ...], str] = {}
+
+    def request_authorisation(self) -> None:
+        """
+        Request authorisation to send user notifications. This method returns
+        immediately but authorisation will only be granted once the user has accepted
+        the prompt. Use :attr:`has_authorisation` to check if we are authorised.
+        """
 
-    def _request_authorisation(self) -> None:
         def on_auth_completed(granted: bool, error: objc_id) -> None:
             if granted:
                 logger.debug("Authorisation granted")
@@ -141,7 +149,8 @@ class CocoaNotificationCenter(DesktopNotifierBase):
         self._did_request_authorisation = True
 
     @property
-    def _has_authorisation(self) -> bool:
+    def has_authorisation(self) -> bool:
+        """Whether we have authorisation to send notifications."""
 
         # get existing notification categories
 
@@ -167,31 +176,26 @@ class CocoaNotificationCenter(DesktopNotifierBase):
 
         return authorized
 
-    def send(self, notification: Notification) -> None:
+    def _send(
+        self,
+        notification: Notification,
+        notification_to_replace: Optional[Notification],
+    ) -> str:
         """
-        Sends a notification.
+        Uses UNUserNotificationCenter to schedule a notification.
 
         :param notification: Notification to send.
+        :param notification_to_replace: Notification to replace, if any.
         """
 
-        if not self.nc:
-            return
-
         if not self._did_request_authorisation:
-            self._request_authorisation()
+            self.request_authorisation()
 
-        if not self._has_authorisation:
-            return
-
-        # Get an internal ID for the notifications. This will recycle an old ID if
-        # we are above the max number of notifications.
-        internal_nid = self._next_nid()
-
-        # Get the old notification to replace, if any.
-        notification_to_replace = self.current_notifications.get(internal_nid)
+        if not self.has_authorisation:
+            raise RuntimeError("Not authorised")
 
         if notification_to_replace:
-            platform_nid = notification_to_replace.identifier
+            platform_nid = str(notification_to_replace.identifier)
         else:
             platform_nid = str(uuid.uuid4())
 
@@ -205,7 +209,9 @@ class CocoaNotificationCenter(DesktopNotifierBase):
         content.title = notification.title
         content.body = notification.message
         content.categoryIdentifier = category_id
-        content.userInfo = {"internal_nid": internal_nid}
+
+        if notification.sound:
+            content.sound = UNNotificationSound.defaultSound
 
         notification_request = UNNotificationRequest.requestWithIdentifier(
             platform_nid, content=content, trigger=None
@@ -214,10 +220,7 @@ class CocoaNotificationCenter(DesktopNotifierBase):
         # Post the notification.
         self.nc.addNotificationRequest(notification_request, withCompletionHandler=None)
 
-        # Store the notification for future replacement and to keep track of
-        # user-supplied callbacks.
-        notification.identifier = platform_nid
-        self.current_notifications[internal_nid] = notification
+        return platform_nid
 
     def _category_id_for_button_names(
         self, button_names: Tuple[str, ...]
@@ -225,6 +228,9 @@ class CocoaNotificationCenter(DesktopNotifierBase):
         """
         Creates and registers a new notification category with the given buttons
         or retrieves an existing one.
+
+        :returns: UNUserNotificationCenter ID for a new or an exiting category. Returns
+            None if a notification category without buttons is requested.
         """
 
         if not button_names:
@@ -272,3 +278,22 @@ class CocoaNotificationCenter(DesktopNotifierBase):
             categories.release()
 
             return category_id
+
+    def _clear(self, notification: Notification) -> None:
+        """
+        Removes a notifications from the notification center
+
+        :param notification: Notification to clear.
+        """
+        self.nc.removeDeliveredNotificationsWithIdentifiers([notification.identifier])
+
+    def _clear_all(self) -> None:
+        """
+        Clears all notifications from notification center
+
+        The method executes asynchronously, returning immediately and removing the
+        notifications on a background thread. This method does not affect any
+        notification requests that are scheduled, but have not yet been delivered.
+        """
+
+        self.nc.removeAllDeliveredNotifications()
