@@ -12,6 +12,7 @@ NSUserNotificationCenter backend for macOS.
 import uuid
 import platform
 import logging
+from typing import Optional
 
 # external imports
 from rubicon.objc import NSObject, ObjCClass, objc_method, py_from_ns  # type: ignore
@@ -36,6 +37,8 @@ NSUserNotificationActivationTypeContentsClicked = 1
 NSUserNotificationActivationTypeActionButtonClicked = 2
 NSUserNotificationActivationTypeAdditionalActionClicked = 4
 
+NSUserNotificationDefaultSoundName = "DefaultSoundName"
+
 
 class NotificationCenterDelegate(NSObject):  # type: ignore
     """Delegate to handle user interactions with notifications"""
@@ -45,16 +48,16 @@ class NotificationCenterDelegate(NSObject):  # type: ignore
         self, center, notification
     ) -> None:
 
-        internal_nid = py_from_ns(notification.userInfo["internal_nid"])
-        notification_info = self.interface.current_notifications[internal_nid]
+        platform_nid = py_from_ns(notification.identifier)
+        py_notification = self.interface._notification_for_nid[platform_nid]
 
         if (
             notification.activationType
             == NSUserNotificationActivationTypeContentsClicked
         ):
 
-            if notification_info.action:
-                notification_info.action()
+            if py_notification.action:
+                py_notification.action()
 
         elif (
             notification.activationType
@@ -62,7 +65,7 @@ class NotificationCenterDelegate(NSObject):  # type: ignore
         ):
 
             button_title = py_from_ns(notification.actionButtonTitle)
-            callback = notification_info.buttons.get(button_title)
+            callback = py_notification.buttons.get(button_title)
 
             if callback:
                 callback()
@@ -78,28 +81,39 @@ class CocoaNotificationCenterLegacy(DesktopNotifierBase):
 
     :param app_name: The name of the app. Does not have any effect because the app
         name is automatically determined from the bundle or framework.
+    :param app_icon: The icon of the app. Does not have any effect because the app
+        icon is automatically determined from the bundle or framework.
+    :param notification_limit: Maximum number of notifications to keep in the system's
+        notification center.
     """
 
-    def __init__(self, app_name: str) -> None:
-        super().__init__(app_name)
+    def __init__(
+        self,
+        app_name: str = "Python",
+        app_icon: Optional[str] = None,
+        notification_limit: Optional[int] = None,
+    ) -> None:
+        super().__init__(app_name, app_icon, notification_limit)
 
         self.nc = NSUserNotificationCenter.defaultUserNotificationCenter
         self.nc_delegate = NotificationCenterDelegate.alloc().init()
         self.nc_delegate.interface = self
         self.nc.delegate = self.nc_delegate
 
-    def send(self, notification: Notification) -> None:
+    def _send(
+        self,
+        notification: Notification,
+        notification_to_replace: Optional[Notification],
+    ) -> str:
         """
-        Sends a notification.
+        Uses NSUserNotificationCenter to schedule a notification.
 
         :param notification: Notification to send.
+        :param notification_to_replace: Notification to replace, if any.
         """
 
-        internal_nid = self._next_nid()
-        notification_to_replace = self.current_notifications.get(internal_nid)
-
         if notification_to_replace:
-            platform_nid = notification_to_replace.identifier
+            platform_nid = str(notification_to_replace.identifier)
         else:
             platform_nid = str(uuid.uuid4())
 
@@ -107,9 +121,14 @@ class CocoaNotificationCenterLegacy(DesktopNotifierBase):
         n.title = notification.title
         n.informativeText = notification.message
         n.identifier = platform_nid
-        n.userInfo = {"internal_nid": internal_nid}
         n.deliveryDate = NSDate.dateWithTimeInterval(0, sinceDate=NSDate.date())
 
+        # store the notification instance for clearing
+        notification._native = n  # type: ignore
+
+        if n.sound:
+            n.soundName = NSUserNotificationDefaultSoundName
+
         if notification.buttons:
             if len(notification.buttons) > 1:
                 logger.debug(
@@ -120,5 +139,21 @@ class CocoaNotificationCenterLegacy(DesktopNotifierBase):
 
         self.nc.scheduleNotification(n)
 
-        notification.identifier = platform_nid
-        self.current_notifications[internal_nid] = notification
+        return platform_nid
+
+    def _clear(self, notification: Notification) -> None:
+        """
+        Removes a notifications from the notification center
+
+        :param notification: Notification to clear.
+        """
+
+        if hasattr(notification, "_native"):
+            self.nc.removeDeliveredNotification(notification._native)  # type: ignore
+
+    def _clear_all(self) -> None:
+        """
+        Clears all notifications from notification center
+        """
+
+        self.nc.removeAllDeliveredNotifications()
