@@ -5,9 +5,10 @@ the platform.
 """
 
 # system imports
+import os
 import platform
 from threading import Lock
-from typing import Type, Optional, Dict, Callable
+from typing import Type, Optional, Dict, Callable, List
 
 # external imports
 from packaging.version import Version
@@ -18,7 +19,12 @@ from .base import NotificationLevel, Notification, DesktopNotifierBase
 
 Impl: Type[DesktopNotifierBase]
 
-if platform.system() == "Darwin":
+if os.environ.get("CI", False):
+    # don't attempt to initialise notification center in CIs such as github actions
+    # this may otherwise lead to segfaults on macOS test runners
+    from .dummy import DummyNotificationCenter as Impl
+
+elif platform.system() == "Darwin":
 
     macos_version, *_ = platform.mac_ver()
 
@@ -31,7 +37,7 @@ elif platform.system() == "Linux":
     from .dbus import DBusDesktopNotifier as Impl
 
 else:
-    from .base import DesktopNotifierBase as Impl
+    from .dummy import DummyNotificationCenter as Impl
 
 
 __all__ = [
@@ -55,13 +61,22 @@ class DesktopNotifier:
         Linux, this should correspond to the application name in a desktop entry. On
         macOS, this argument is ignored and the app is identified by the bundle ID of
         the sending program (e.g., Python).
+    :param app_icon: Default icon to use for notifications. This may be be either an URI
+        (file:// is the only URI schema supported right now) or a name in a
+        freedesktop.org-compliant icon theme. On macOS, this argument is ignored and the
+        app icon is identified by the bundle ID of the sending program (e.g., Python).
     :param notification_limit: Maximum number of notifications to keep in the system's
         notification center. This may be ignored by some implementations.
     """
 
-    def __init__(self, app_name: str = "Python", notification_limit: int = 5) -> None:
+    def __init__(
+        self,
+        app_name: str = "Python",
+        app_icon: Optional[str] = None,
+        notification_limit: Optional[int] = None,
+    ) -> None:
         self._lock = Lock()
-        self._impl = Impl(app_name, notification_limit)
+        self._impl = Impl(app_name, app_icon, notification_limit)
 
     def send(
         self,
@@ -71,7 +86,8 @@ class DesktopNotifier:
         icon: Optional[str] = None,
         action: Optional[Callable] = None,
         buttons: Optional[Dict[str, Callable]] = None,
-    ) -> None:
+        sound: bool = False,
+    ) -> Notification:
         """
         Sends a desktop notification. Some arguments may be ignored, depending on the
         backend.
@@ -88,8 +104,38 @@ class DesktopNotifier:
             by some implementations.
         :param buttons: A dictionary with button names and callbacks to show in the
             notification. This is ignored by some implementations.
+        :param sound: A sound to play when showing the notification. Can be a predefined
+            sound or the path to a sound file.
+
+        :returns: The scheduled notification instance.
         """
-        notification = Notification(title, message, urgency, icon, action, buttons)
+        notification = Notification(
+            title, message, urgency, icon, action, buttons, sound
+        )
 
         with self._lock:
             self._impl.send(notification)
+
+        return notification
+
+    @property
+    def current_notifications(self) -> List[Notification]:
+        """A list of all currently displayed notifications for this app"""
+        return self._impl.current_notifications
+
+    def clear(self, notification: Notification) -> None:
+        """
+        Removes the given notification from the notification center.
+
+        :param notification: Notification to clear.
+        """
+        with self._lock:
+            self._impl.clear(notification)
+
+    def clear_all(self) -> None:
+        """
+        Removes all currently displayed notifications for this app from the notification
+        center.
+        """
+        with self._lock:
+            self._impl.clear_all()
